import java.util.*;

import static org.gecode.Gecode.*;
import static org.gecode.GecodeEnumConstants.*;

import org.gecode.*;

// This class implements a new variable type on top of finite set variables:
// Finite binary relations

class RelVar {
    
    // A binary relation is implemented as an array of finite set variables.
    // r[i] is the relational image of r at position i.
    public VarArray<SetVar> r;

    static final IntSet empty = new IntSet(1,0);

    // Allocating a binary relation variable creates n finite set variables,
    // all initialized with empty lower bound and an upper bound of 0,...,n-1
    public RelVar(Space home, int n) {
        r = new VarArray<SetVar>(home, n, SetVar.class,
                                 empty, new IntSet(0,n-1));
    }

    public RelVar copy(Space home, Boolean share) {
        RelVar nr = new RelVar(home, r.size());
        nr.r = new VarArray<SetVar>(home, share, r);
        return nr;
    }

    // Print as a set of pairs
    public String toString() {
        String ret = "";
        int i = 0;
        for (SetVar sr : r) {
            for (Range rng : new SetVarGlbRanges(sr)) {
                for (int j=rng.min(); j<=rng.max(); j++) {
                    ret += "("+i+","+j+")\n";
                }
            }
            i++;
        }

        return ret;
    }

    // Example constraint: this relation is the union of r1 and r2,
    // defined pointwise
    public void union(Space home, RelVar r1, RelVar r2) {
        for (int i=0; i<r.size(); i++) {
            rel(home, r1.r.get(i), SOT_UNION, r2.r.get(i), SRT_EQ, r.get(i));
        }
    }

    // Example constraint: this relation is the intersection of r1 and r2,
    // defined pointwise
    public void intersection(Space home, RelVar r1, RelVar r2) {
        for (int i=0; i<r.size(); i++) {
            rel(home, r1.r.get(i), SOT_INTER, r2.r.get(i), SRT_EQ, r.get(i));
        }
    }

    // TODO: Implement that this relation is a sub-relation of r1
    public void subrelation(Space home, RelVar r1) {
        // TODO: insert code here
    }

    // TODO: Implement that this relation is equal to r1
    public void equal(Space home, RelVar r1) {
        // TODO: insert code here
    }

    // TODO: Implement that this relation is disjoint from r1
    public void disjoint(Space home, RelVar r1) {
        // TODO: insert code here
    }

    // TODO: Implement that this relation is the composition r1 o r2
    public void composition(Space home, RelVar r1, RelVar r2) {
        // TODO: insert code here
    }
    
    
    // TODO: Implement that this relation is the converse of r1
    public void converse(Space home, RelVar r1) {
        // TODO: insert code here
    }
    
    // TODO: Implement that this relation is reflexive
    public void reflexive(Space home) {
        // TODO: insert code here
    }
    
    // TODO: Implement that this relation is transitive
    public void transitive(Space home) {
        // TODO: insert code here
    }

    // TODO: Implement that this relation is anti-symmetric
    public void antisymmetric(Space home) {
        // TODO: Implement that this relation is anti-symmetric
    }

    // TODO: Implement that this relation is total on s
    public void total(Space home, SetVar s) {
        // TODO: Implement that this relation is anti-symmetric
    }

    // Make life easier: branch on all set variables
    public void branch(Space home) {
        Gecode.branch(home, r, SETBVAR_MIN_CARD, SETBVAL_MIN);
    }
    
}
